############################################################################
#                                                                          #
#  The Logic and Methodology of "Necessary but not Sufficient Causality":  #
#  A Comment on Necessary Condition Analysis (NCA)                         #
#                                                                          #
#  Alrik Thiem                                                             #
#                                                                          #
#  5 June 2018                                                             #
#                                                                          #
############################################################################

#------------------------------------------------------------------------------#
# load packages                                                                #
#------------------------------------------------------------------------------#
library(NCA)
library(QCApro)

#------------------------------------------------------------------------------#
# NCA: A Tool for Identifying Necessary but Insufficient Causes?               #
#------------------------------------------------------------------------------#

# construct admission success dataset
#------------------------------------

# create a matrix with all possible combinations of conditions that include
# all factors (including the endogenous factor AD)
# GRE: GRE test score; h = 0: high; m = 1: medium; l = 2: low
# TOE: TOEFL score; l = 0: low; m = 1: medium; h = 2: high
# ML : motivation letter; n = 0: not strong; s = 1: strong
# REC: recommendation letter; n = 0: not positive; p = 1: positive
# UGD: reputation of undergraduate degree-awarding university; n = 0: no top
#      reputation; t = 1: top reputation
# SEX: sex; f = 0: female; m = 1: male
# PAR: parents have academic background; n = 0: no; a = 1: yes
# FD : parents have made significant financial donation; n = 0: no; s = 1: yes
# AD : admission decision; p = 1: positive; n = 0: negative

minmat <- as.data.frame(mintermMatrix(c(3, 3, rep(2,7))))

dimnames(minmat) <- list(as.character(seq(3*3*2^7)), c("GRE", "TOE", "ML", "REC",
 "UGD", "SEX", "PAR", "FD", "AD"))

# create R's dataset by filtering all rows from minmat that comply with the
#
#                      path (I)              path (II)
# multi-path DGS: GRE{h}*ML{s}*UGD{t} + GRE{h}*PAR{a}*FD{s}  <=> AD{p};
# let path (I) be the unique reason for admission about six times as often as
# path (II), and let the admission success rate be about 12%

dat1.nw <- with(minmat,
 minmat[(GRE == 0 & ((ML == 1 & UGD == 1) | (PAR == 1 & FD == 1))) == AD,])

dat1 <- rbind(
  dat1.nw,
  dat1.nw[rep(rownames(with(dat1.nw, dat1.nw[GRE == 0 & ML == 1 & UGD == 1 &
                                            (PAR == 0 | FD == 0),])),
              each = 5), ],
  dat1.nw[rep(rownames(with(dat1.nw, dat1.nw[AD == 0,])),
              each = 3), ]
)

# NCA analyses
#--------------------------------------

# factor labels
fcts <- c("GRE", "TOE", "ML", "REC", "UGD", "SEX", "PAR", "FD")

# ceiling techniques
cls <- c("lh", "cols", "qr", "ce_vrs", "cr_vrs", "ce_fdh", "cr_fdh")

# analysis
nca1.dat1 <- nca(dat1, x = fcts, y = "AD", ceilings = cls)
nca1.dat1
graphics.off()

# footnote 7: changes in the admission success rate and the distribution of
# applicants across the two paths to admission success, both of which are
# unrelated to the DGS itself, will lead Quantile Regression (QR) to also 
# produce only null results
nca1.dat1 <- nca(dat1.nw, x = fcts, y = "AD", ceilings = cls)
nca1.dat1
graphics.off()

# can NCA separate causes from conditions?
# FUD: field in which the applicant received undergraduate degree;
# m = 1: management or sociology; o = 0: other discipline;
# FUD{m} is a necessary condition but no necessary cause of admission success
dat2 <- cbind(dat1, FUD = 1)
# one applicant who was not admitted (constants pose problems for NCA)
dat2$FUD[1] <- 0

nca2.dat2 <- nca(dat2, x = c(fcts, "FUD"), y = "AD", ceilings = cls)
nca2.dat2
graphics.off()


#------------------------------------------------------------------------------#
# QCA: A Method for Identifying INUS Causality                                 #
#------------------------------------------------------------------------------#

tt.dat1 <- truthTable(dat1, outcome = "AD{1}")

# QCA recovers model m1
sol.dat1 <- eQMC(tt.dat1, details = TRUE)
sol.dat1